@@ -29,12 +29,12 @@ let () =
     (* gen#fill buf 0 blen; *)
     (try
        assert (alsa_out#wait 1000);
-       let w = alsa_out#write buf in
+       let w = alsa_out#write buf 0 blen in
        Printf.printf "Wrote: %d\n%!" w
      with Alsa.Buffer_xrun as e ->
        alsa_out#recover e;
-       ignore (alsa_out#write buf));
-    let _ = alsa_in#read buf in
+       ignore (alsa_out#write buf 0 blen));
+    let _ = alsa_in#read buf 0 blen in
     (* Printf.printf "Read: %d\n%!" r *) ()
   done;
   alsa_in#close;
