@@ -4,13 +4,14 @@ external is_big_endian : unit -> bool = "ocaml_mm_is_big_endian"
 
 let () =
   C.main ~name:"mm" (fun c ->
-      let has_aligned_alloc =
+      let has_posix_memalign =
         C.c_test c
           {|
         #include <stdlib.h>
 
         int main() {
-          char *data = aligned_alloc(16, 4096);
+          void *data;
+          posix_memalign(&data, 16, 4096);
           return 0;
         }
       |}
@@ -56,7 +57,7 @@ let () =
       C.C_define.gen_header_file c ~fname:"config.h"
         [
           ("BIGENDIAN", Switch (is_big_endian ()));
-          ("HAS_ALIGNED_ALLOC", Switch has_aligned_alloc);
+          ("HAS_POSIX_MEMALIGN", Switch has_posix_memalign);
           ("HAS_MEMALIGN", Switch has_memalign);
           ("HAS_MAX_ALIGN_T", Switch has_max_align_t);
           ("HAS_CAML_INTERNALS", Switch has_caml_internals);
