@@ -60,10 +60,62 @@ module Pixel : sig
   val rgb_of_yuv : yuv -> rgb
 end
 
+module Point : sig
+  type t = int * int
+
+  val min : t -> t -> t
+  val max : t -> t -> t
+  val lt : t -> t -> bool
+  val le : t -> t -> bool
+  val neg : t -> t
+end
+
+module Fraction : sig
+  type t = int * int
+
+  val min : t -> t -> t
+end
+
 module Draw : sig
   val line : (int -> int -> unit) -> int * int -> int * int -> unit
 end
 
+(** Operation on bitmaps, which are black and white images. *)
+module Bitmap : sig
+  (** A bitmap. *)
+  type t
+
+  type bitmap = t
+
+  val create : int -> int -> t
+
+  val width : t -> int
+
+  val height : t -> int
+
+  val get_pixel : t -> int -> int -> bool
+
+  val set_pixel : t -> int -> int -> bool -> unit
+
+  val scale : t -> t -> unit
+
+  (** Operations on bitmap fonts. *)
+  module Font : sig
+    (** A font. *)
+    type t
+
+    (** Our native font. *)
+    val native : t
+
+    (** Height in pixels of characters. *)
+    val height : t -> int
+
+    (** Render text with given font, at given size (height of characters in
+        pixels). *)
+    val render : ?font:t -> ?size:int -> string -> bitmap
+  end
+end
+
 (** Operations on images stored in RGB8 format, ie RGB channels, one byte each. *)
 module RGB8 : sig
   (** Operations on colors. *)
@@ -213,11 +265,16 @@ end
 (** Operations on images stored in YUV420 format, ie one luma (Y) and two chrominance (U and V) channels. *)
 module YUV420 : sig
   (** An image in YUV420 format. *)
-  type t
+  type t = ImageYUV420.t
+
+  (** Create an image with given width, height, alpha channel, Y (with given
+      stride) U and V (with given stride). The strides of U and V are the same,
+      the stride of the alpha channel is the same as Y. *)
+  val make :
+    int -> int -> ?alpha:Data.t -> Data.t -> int -> Data.t -> Data.t -> int -> t
 
-  val make : int -> int -> Data.t -> int -> Data.t -> Data.t -> int -> t
   val make_data : int -> int -> Data.t -> int -> int -> t
-  val create : ?y_stride:int -> ?uv_stride:int -> int -> int -> t
+  val create : ?blank:bool -> ?y_stride:int -> ?uv_stride:int -> int -> int -> t
 
   (** Ensure that the image has an alpha channel. *)
   val ensure_alpha : t -> unit
@@ -230,6 +287,8 @@ module YUV420 : sig
   val of_RGB24_string : string -> int -> t
   val of_RGBA32 : RGBA32.t -> t
   val to_RGBA32 : t -> RGBA32.t
+  val of_bitmap : ?fg:Pixel.rgba -> ?bg:Pixel.rgba -> Bitmap.t -> t
+  val to_BMP : t -> string
   val of_PPM : string -> t
 
   (** Width of an image. *)
@@ -260,21 +319,59 @@ module YUV420 : sig
   val copy : t -> t
   val blit_all : t -> t -> unit
   val blit : t -> t -> unit
+
+  (** Scale one image in order to fill the other. By default, proportions are
+      not preserved. *)
   val scale : ?proportional:bool -> t -> t -> unit
+
   val blank_all : t -> unit
 
-  (** Add the fist image to the second. *)
+  (** Add the fist image to the second at given offset. *)
   val add : t -> ?x:int -> ?y:int -> t -> unit
 
   val blank : t -> unit
   val fill : t -> Pixel.yuv -> unit
+
+  (** Flip image horizontally. *)
+  val hmirror : t -> unit
+
+  (** Whether the image is opaque (it has no transparent or semi-transparent
+      pixel). *)
+  val is_opaque : t -> bool
+
+  (** Optimize the α channel by removing it in the case the image is opaque. *)
+  val optimize_alpha : t -> unit
+
   val fill_alpha : t -> int -> unit
   val disk_alpha : t -> int -> int -> int -> unit
+  val alpha_of_color : t -> Pixel.yuv -> int -> unit
 
-  (* [box_alpha img x y width height alpha] Set alpha value
-     on a given image box. *)
+  (** Takes a reference image and an image, and make similar portions
+      transparent on the second (the last parameter controls the tolerance). This
+      is useful to make bluescreens withtout bluescreens. *)
+  val alpha_of_sameness : t -> t -> int -> unit
+
+  (** [alpha_of_diff prev curr level speed] takes a previous image and a current
+      image and make parts of the current image more transparent if they were the
+      same. [level] is the distance at which we consider two colors to be the
+      same and [speed] is the inverse of the convergence speed. *)
+  val alpha_of_diff : t -> t -> int -> int -> unit
+
+  (** [box_alpha img x y width height alpha] sets alpha value on a given image
+      box. *)
   val box_alpha : t -> int -> int -> int -> int -> float -> unit
+
+  (** Remove alpha channel and set it as Y channel. Useful to inspect the alpha
+      channel. *)
+  val alpha_to_y : t -> unit
+
   val randomize : t -> unit
+  val rotate : t -> int -> int -> float -> t -> unit
+
+  (** Fill the image with a gradient. It takes as argument the (U,V) at pixel
+     (0,0), at pixel (xmax,0) and at pixel (0,ymax). *)
+  val gradient_uv : t -> int * int -> int * int -> int * int -> unit
+
   val get_pixel_y : t -> int -> int -> int
   val get_pixel_u : t -> int -> int -> int
   val get_pixel_v : t -> int -> int -> int
@@ -385,3 +482,123 @@ module Generic : sig
   val convert :
     ?proportional:bool -> ?scale_kind:RGBA32.Scale.kind -> t -> t -> unit
 end
+
+(** Type of image module expected to build canvas. *)
+module type CanvasImage = sig
+  (** An image. *)
+  type t
+
+  (** Width of the image. *)
+  val width : t -> int
+
+  (** Height of the image. *)
+  val height : t -> int
+
+  (** Size of the image in bytes. *)
+  val size : t -> int
+
+  (** Create an image with given dimensions. *)
+  val create : int -> int -> t
+
+  (** Clear the image. *)
+  val blank : t -> unit
+
+  (** Create a copy of the image. *)
+  val copy : t -> t
+
+  (** Add source image at given offset to the target image. *)
+  val add : t -> ?x:int -> ?y:int -> t -> unit
+
+  (** Whether the image has alpha channel. *)
+  val has_alpha : t -> bool
+
+  (** Fill the alpha channel of the image. *)
+  val fill_alpha : t -> int -> unit
+
+  (** Set the RGBA value of a pixel. *)
+  val set_pixel_rgba : t -> int -> int -> Pixel.rgba -> unit
+
+  (** Fill the image with random data. *)
+  val randomize : t -> unit
+
+  (** Scale the source image to the destination image. *)
+  val scale : t -> t -> unit
+end
+
+(** Canvas of images, i.e. formal sums of images of various dimensions with
+    various offsets. *)
+module Canvas (I : CanvasImage) : sig
+  (** A canvas. *)
+  type t
+
+  (** Create an empty canvas. *)
+  val create : int -> int -> t
+
+  (** Create a canvas containing a given image. Negative dimensions are
+      ignored, default ones are those of the image. *)
+  val make : ?width:int -> ?height:int -> ?x:int -> ?y:int -> I.t -> t
+
+  (** Width of the image. *)
+  val width : t -> int
+
+  (** Height of the image. *)
+  val height : t -> int
+
+  (** Change the width and height of the viewport of the canvas. *)
+  val viewport : ?x:int -> ?y:int -> int -> int -> t -> t
+
+  (** Size of a canvas in bytes. *)
+  val size : t -> int
+
+  (** Add two canvas. The first one is on top of the second one. *)
+  val add : t -> t -> t
+
+  (** Whether the canvas covers the whole area with images (this function is
+      imprecise: it might have false negatives). *)
+  val covering : t -> bool
+
+  (** Render the canvas as an image. If [fresh] is set to true, the resulting
+      can be modified in place. If [transparent] is set to true, the non-covered
+      portions are made transparent. *)
+  val render : ?fresh:bool -> ?transparent:bool -> t -> I.t
+
+  (** Rendered canvas. *)
+  val rendered : ?transparent:bool -> t -> t
+
+  (** Map a function on the underlying image. This of course triggers a render
+      of the canvas. *)
+  val map : (I.t -> I.t) -> t -> t
+
+  (** Execute a function on the rendering of the canvas. *)
+  val iter : (I.t -> unit) -> t -> t
+
+  (** Translate the image. *)
+  val translate : int -> int -> t -> t
+
+  (** Bouding box (smallest box enclosing all images contained in the canavs):
+      returns the lower-left corner and the dimensions. *)
+  val bounding_box : t -> (int * int) * (int * int)
+
+  (** Scale the image proportionally by fractional coefficients with given
+      numerator and denominator in the x and y directions. The viewport is left
+      untouched. The [scaler] parameter can be specified in order to use a
+      particular function to scale individual images. The viewport is left
+      unchanged. *)
+  val scale : ?scaler:(I.t -> I.t -> unit) -> Fraction.t -> Fraction.t -> t -> t
+
+  (** Resize the image, scaling and changing the viewport. *)
+  val resize :
+    ?proportional:bool -> ?scaler:(I.t -> I.t -> unit) -> int -> int -> t -> t
+
+  module Draw : sig
+    (** Draw a line (the result is typically added to another image). *)
+    val line : Pixel.rgba -> int * int -> int * int -> t
+  end
+end
+
+module CanvasYUV420 : module type of Canvas (struct
+  include ImageYUV420
+
+  let create w h = create w h
+  let scale = scale ~proportional:false
+end)
